from huawei_solar_tools.utils.connection import create_modbus_tcp_client
from huawei_solar_tools.utils.register_interpreter import convert_registers_to_readable


class Device:
    """
    Base class for all Huawei solar devices.
    Handles common functionality such as Modbus communication and register operations.
    """
    def __init__(self, host, port, device_id, registers):
        """
        Initialize the device.

        :param host: IP address of the Modbus server.
        :param port: Port of the Modbus server.
        :param device_id: Modbus unit ID of the device.
        :param registers: Dictionary of register mappings for the device.
        """
        self.host = host
        self.port = port
        self.device_id = device_id
        self.registers = registers
        self.client = None

    async def connect(self):
        """
        Establish a Modbus TCP connection.
        """
        self.client = await create_modbus_tcp_client(self.host, self.port, self.device_id)
        if not self.client:
            raise ConnectionError(f"Failed to connect to device {self.device_id} at {self.host}:{self.port}")

    def disconnect(self):
        """
        Close the Modbus TCP connection.
        """
        if self.client:
            self.client.close()
            self.client = None

    async def read_register_by_name(self, name):
        """
        Read a register by its name.

        :param name: Name of the register to read.
        :return: Decoded value of the register.
        """
        if name not in self.registers:
            raise ValueError(f"Register '{name}' not found in device register mappings.")

        reg_info = self.registers[name]
        address = reg_info['address']
        quantity = reg_info['quantity']

        if not self.client:
            raise ConnectionError("Device is not connected.")

        try:
            registers = self.client.read_holding_registers(address, quantity)
            if not registers:
                raise ValueError(f"Failed to read register '{name}' at address {address}.")
            return convert_registers_to_readable(registers, reg_info)
        except Exception as e:
            raise RuntimeError(f"Error reading register '{name}': {e}")

    async def read_raw_register(self, address, quantity):
        """
        Read raw registers directly by address and quantity.

        :param address: Starting address of the registers to read.
        :param quantity: Number of registers to read.
        :return: Raw register values.
        """
        if not self.client:
            raise ConnectionError("Device is not connected.")

        try:
            registers = self.client.read_holding_registers(address, quantity)
            if not registers:
                raise ValueError(f"Failed to read registers at address {address}.")
            return registers
        except Exception as e:
            raise RuntimeError(f"Error reading raw registers at address {address}: {e}")

    async def read_all_registers(self):
        """
        Read all registers defined in the device's register mappings.

        :return: List of tuples with register name, value, and unit.
        """
        results = []
        for key, reg_info in self.registers.items():
            address = reg_info['address']
            quantity = reg_info['quantity']
            unit = reg_info.get('unit', 'N/A')

            try:
                # Read the registers
                registers = self.client.read_holding_registers(address, quantity)
                if registers:
                    # Convert the registers to a readable value
                    value = convert_registers_to_readable(registers, reg_info)
                    results.append((key, value, unit))
                else:
                    results.append((key, None, unit))
            except Exception as e:
                print(f"Error processing register {key} at address {address}: {e}")
                results.append((key, f"Error: {e}", unit))
        return results

    def is_connected(self):
        """
        Check if the Modbus client is connected.

        :return: True if connected, False otherwise.
        """
        return self.client is not None

    def get_device_id(self):
        """
        Get the device ID.

        :return: The device ID.
        """
        return self.device_id
