import asyncio
import os
import matplotlib.pyplot as plt
import sys
from dotenv import load_dotenv
from huawei_solar_tools.devices.emi import EMI
import registers.emi_register_names as ern

async def main():
    # Load environment variables
    load_dotenv("huawei_conf.env")
    host = os.getenv("SMARTLOGGER_IP")
    port = int(os.getenv("SMARTLOGGER_PORT"))
    emi_ids = [int(i) for i in os.getenv("EMI_IDS").split(",")]  # Support multiple emi IDs

    for device_id in emi_ids:
        # Set up individual report file for each emi
        script_dir = os.path.dirname(os.path.abspath(__file__))
        reports_dir = os.path.join(script_dir, "emi_reports")
        os.makedirs(reports_dir, exist_ok=True)
        report_file = os.path.join(reports_dir, f"emi_{device_id}_report.txt")

        with open(report_file, "w") as report:
            sys.stdout = report

            print(f"Connecting to emi {device_id}...")

            # Initialize the Inverter
            emi = EMI(host, port, device_id)

            try:
                # Test connection
                await emi.connect()
                print("Connection successful.")

                # Test connection status
                print(f"Is emi {device_id} connected? {emi.is_connected()}")

                # Test getting device ID
                print(f"Inverter ID: {emi.get_device_id()}")

                # Test reading all registers
                results = await emi.read_all_registers()
                print("All Registers:")
                for i in results:
                    print(i)

                # Test reading a specific register (grid frequency)
                freq = await emi.read_raw_register(40031, 1)
                print(f"Grid Frequency: {freq[0] / 100} Hz")

                # Test reading a specific register by name (Serial Number)
                serial_number = await emi.read_register_by_name(ern.WIND_SPEED)
                print(f"Wind Speed: {serial_number}")
                
            except Exception as e:
                print(f"Error processing emi {device_id}: {e}")

            finally:
                # Test disconnection
                emi.disconnect()
                print(f"Disconnected from emi {device_id}.")

if __name__ == "__main__":
    asyncio.run(main())
