import asyncio
import os
import matplotlib.pyplot as plt
import sys
from dotenv import load_dotenv
from huawei_solar_tools.devices.inverter import Inverter
import huawei_solar_tools.registers.inverter_register_names as irn

async def main():
    # Load environment variables
    load_dotenv("huawei_conf.env")
    host = os.getenv("SMARTLOGGER_IP")
    port = int(os.getenv("SMARTLOGGER_PORT"))
    inverter_ids = [int(i) for i in os.getenv("INVERTER_IDS").split(",")]  # Support multiple inverter IDs

    for device_id in inverter_ids:
        # Set up individual report file for each inverter
        script_dir = os.path.dirname(os.path.abspath(__file__))
        reports_dir = os.path.join(script_dir, "inverter_reports")
        os.makedirs(reports_dir, exist_ok=True)
        report_file = os.path.join(reports_dir, f"inverter_{device_id}_report.txt")

        with open(report_file, "w") as report:
            sys.stdout = report

            print(f"Connecting to inverter {device_id}...")

            # Initialize the Inverter
            inverter = Inverter(host, port, device_id)

            try:
                # Test connection
                await inverter.connect()
                print("Connection successful.")

                # Test connection status
                print(f"Is inverter {device_id} connected? {inverter.is_connected()}")

                # Test getting device ID
                print(f"Inverter ID: {inverter.get_device_id()}")

                # Test interpreting alarms
                alarms = await inverter.interpret_alarms()
                print("Interpreted Alarms:")
                for alarm_type, alarm_details in alarms.items():
                    print(f"{alarm_type}: {alarm_details}")

                # Test interpreting states
                states = await inverter.interpret_states()
                print("Interpreted States:")
                print(states)

                # Test reading all registers
                results = await inverter.read_all_registers()
                print("All Registers:")
                for i in results:
                    print(i)

                # Test reading a specific register (grid frequency)
                freq = await inverter.read_raw_register(32085, 1)
                print(f"Grid Frequency: {freq[0] / 100} Hz")

                # Test reading a specific register by name (Serial Number)
                serial_number = await inverter.read_register_by_name(irn.SN)
                print(f"Serial Number: {serial_number}")

                # Test interpreting cosφ-P/Pn curve, Q-U curve, and PF-U curve
                curve_1 = await inverter.interpret_cos_phi_p_pn_curve()
                curve_2 = await inverter.interpret_qu_curve()
                curve_3 = await inverter.interpret_pf_u_curve()

                # Plot and save curves
                plot_and_save_curve(curve_1, f"cosφ-P/Pn Curve (Inverter {device_id})", "P/Pn (%)", "cosφ", "P/Pn", "cosφ", f"cos_phi_p_pn_curve_{device_id}.png")
                plot_and_save_curve(curve_2, f"Q-U Curve (Inverter {device_id})", "U (%)", "Q (kVar)", "U", "Q", f"qu_curve_{device_id}.png")
                plot_and_save_curve(curve_3, f"PF-U Curve (Inverter {device_id})", "U (%)", "PF", "U", "PF", f"pf_u_curve_{device_id}.png")

                # Test get_startup_time
                startup_time = await inverter.get_startup_time()
                print(f"startup_time: {startup_time}, type = {type(startup_time)}")
                startup_time = await inverter.get_startup_time(iso=True)
                print(f"startup_time: {startup_time}, type = {type(startup_time)}")
                # Test get_startup_time
                shutdown_time = await inverter.get_shutdown_time()
                print(f"shutdown_time: {shutdown_time}, type = {type(shutdown_time)}")
                shutdown_time = await inverter.get_shutdown_time(iso = True)
                print(f"shutdown_time: {shutdown_time}, type = {type(shutdown_time)}")
                
                #Test interpret_grid_code
                print(f"Grid Code: {await inverter.interpret_grid_code()}")
                
            except Exception as e:
                print(f"Error processing inverter {device_id}: {e}")

            finally:
                # Test disconnection
                inverter.disconnect()
                print(f"Disconnected from inverter {device_id}.")

def plot_and_save_curve(curve, title, x_label, y_label, x_key, y_key, filename):
    """
    Plot and save a curve to the tests/plots directory.
    """
    if not curve:
        print(f"No data to plot for {title}.")
        return

    x = [point[x_key] for point in curve]
    y = [point[y_key] for point in curve]

    plt.figure(figsize=(8, 6))
    plt.plot(x, y, marker='o', linestyle='-', label=title)
    plt.title(title)
    plt.xlabel(x_label)
    plt.ylabel(y_label)
    plt.grid(True)
    plt.legend()
    plt.tight_layout()

    # Save the plot to the tests/plots directory
    plots_dir = os.path.join(os.path.dirname(__file__), "plots")
    os.makedirs(plots_dir, exist_ok=True)
    plot_path = os.path.join(plots_dir, filename)

    plt.savefig(plot_path)
    plt.close()
    print(f"Plot saved to: {plot_path}")



if __name__ == "__main__":
    asyncio.run(main())
